@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2008-2012 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -127,6 +127,9 @@ class Variable:
     # fill value is true
     untrusted = False
     NONE,CHECK,UNCOMPAT,HUMAN = range(4)
+    # при установке значения переменной устанавливает значение
+    # указанной переменной
+    alias = ""
 
     def __init__(self,parent=None,**kwargs):
         """
@@ -200,6 +203,10 @@ class Variable:
         """Select by datavars"""
         return self.parent.Select(selField,zipVars=self.ZipVars,**kwargs)
 
+    def getRemoteInfo(self, envFile):
+        """getRemoteInfo by datavars"""
+        return self.parent.getRemoteInfo(envFile)
+
     def ZipVars(self,*argvVarNames):
         """
         Get zipped values of variables specified by list 'argvVarNames'
@@ -356,6 +363,8 @@ class Variable:
             value = "on" if self.isTrue(value) else "off"
         if "bool" in self.type and "list" in self.type:
             return map(convert,value)
+        if "int" in self.type:
+            return str(value)
         return value
 
     def raiseNothingValue(self):
@@ -366,16 +375,21 @@ class Variable:
 
     def checkType(self,value):
         """Check value for type"""
+        if "int" in self.type:
+            if value and not value.isdigit():
+                raise VariableError(
+                    _("The value of variable '{varname}' must be integer"
+                            ).format(varname=self.label or self.name))
         if "list" in self.type:
             if not type(value) in (list,tuple):
                 raise VariableError(
-                    _("Value for {varname} may be '{vartype}' only").format(
+                    _("Value for variable '{varname}' may be '{vartype}' only").format(
                     varname=self.label or self.name,
                     vartype="list"))
-            error = _("Values for {varname} may be {vartype} only")
+            error = _("Values for variable '{varname}' may be '{vartype}' only")
         else:
             value = repeat(value,1)
-            error = _("Value for {varname} may be {vartype} only")
+            error = _("Value for variable '{varname}' may be '{vartype}' only")
         if "string" in self.type:
             value, valuecopy = tee(value,2)
             for v in (x for x in valuecopy if not type(x) in (str,unicode)):
@@ -385,6 +399,8 @@ class Variable:
             return
         if "choice" in self.type:
             choiceVal = self.choice()
+            tipChoice = map(lambda x:'"%s"(%s)'%(x[1],x[0]) \
+                            if type(x) in (list,tuple) else str(x), choiceVal)
             if choiceVal and type(choiceVal[0]) in (tuple,list):
                 choiceVal = [x[0] for x in choiceVal]
         if not "list-list" in self.type:
@@ -397,14 +413,14 @@ class Variable:
                         name = self.label or self.name
                         if not choiceVal:
                             self.raiseNothingValue()
-                        self.raiseWrongChoice(name,choiceVal,val,error)
+                        self.raiseWrongChoice(name,tipChoice,val,error)
 
     def raiseWrongChoice(self,name,choiceVal,value,error):
         raise VariableError(error.format(
                             varname=name,
                             vartype=formatListOr(choiceVal)))
 
-    def setValue(self,value):
+    def setValue(self,value,force=False):
         """
         Standard action for set value
         """
@@ -412,25 +428,29 @@ class Variable:
         self.wasSet  = True
         self.invalid = False
         # run check
-        self._check()
+        if not force:
+            self._check()
 
     def _set(self,value,force=False):
         """
         Standard inner method for setting value for variable.
         """
         # runc check choice
-        try:
-            self.modeGet = Variable.CHECK
-            self.checkType(value)
-        finally:
-            self.modeGet = Variable.NONE
+        if type(value) == int:
+            value = str(value)
+        if not force:
+            try:
+                self.modeGet = Variable.CHECK
+                self.checkType(value)
+            finally:
+                self.modeGet = Variable.NONE
         # if value change
         if value != self.value or type(value) in (list,tuple):
             self.invalidate(True)
         if hasattr(self.value,"close"):
             self.value.close()
         # run set value
-        self.setValue(value)
+        self.setValue(value,force)
 
     def _check(self,value=None):
         """
@@ -582,7 +602,7 @@ class TableVariable(Variable):
             "{fieldname} in variable {variablename}").format(
             fieldname=fieldname,variablename=variablename))
 
-    def setValue(self,value):
+    def setValue(self,value,force=False):
         value = self.set(value)
         self.untrusted = True
         oldvalue = self.Get(self.name)
@@ -610,7 +630,7 @@ class TableVariable(Variable):
         # if table with writable index field then replace all table
         error = []
         if self.__isIndexWritable():
-            if any(value):
+            if not force and any(value):
                 self.parent.Check(self.source[0],zip(*value)[0])
             oldvalue = newval
             try:
@@ -1030,6 +1050,15 @@ class DataVars(SimpleDataVars):
     def GetBool(self,varname):
         return Variable.isTrue(self.Get(varname))
 
+    def GetInteger(self,varname):
+        """
+        Получить целочисленное значение
+        """
+        try:
+            return int(self.Get(varname))
+        except ValueError:
+            return 0
+
     def isFromIni(self,varname):
         """
         Check what value get from ini file
@@ -1160,7 +1189,9 @@ class DataVars(SimpleDataVars):
         """
         varObj = self.getInfo(varname)
         if force or varObj.mode != READONLY:
-            varObj._set(varvalue)
+            varObj._set(varvalue,force)
+            if varObj.alias:
+                self.Set(varObj.alias,varObj.value,force)
         else:
             raise DataVarsError(
                 _("Attempt to rewrite readonly variable %s")%
@@ -1223,7 +1254,9 @@ class DataVars(SimpleDataVars):
             if writeVal != config.getVar(header,varname):
                 if not config.setVar(header,{varname: writeVal}):
                     raise DataVarsError(
-                        _("Unable to write variable %s")%varname)
+                        _("Unable to write variable {varname} into "
+                          "'{location}'").format(varname=varname,
+                                                        location=location))
             return True
         else:
             raise DataVarsError(_("Unable to find alias '%s' for the "
@@ -1333,22 +1366,27 @@ class DataVars(SimpleDataVars):
                 varObj.value.close()
 
     def addGroup(self,groupName,normal=(),expert=(),brief=(),next_label=None,
-                 brief_force=(), hide=(), expert_label=None,image=''):
+                 brief_force=(), hide=(), expert_label=None,image='',
+                 custom_buttons=()):
         """
-        Add group by name, normal, expert and next_label
+        Добавить группу переменных для метода
 
-        name - variable name
-        normal - variable display as normal variable
-        expert - variable display in expert mode
-        hide - hide variable in brief
-        image - image (GUI)
-        brief - variable display in brief if it is compatible (at example
-                hide os_migrate_data and display os_migrate_user for brief
-                display)
-        brief_force - display variable even if it is uncompatible (at example
-                      os_location_data with using autopartition)
-        expert_label - replace standard open expert variables label (GUI)
-        next_label - label on button for next step (GUI)
+        groupName - название группы
+        normal - регулярные настройки
+        expert - расширенные настройки
+        hide -   кортеж имен переменных, которые необходимо скрыть в brief
+        image -  изображение
+        brief -  кортеж имен переменных, значение которых необходимо отобразите
+                 в выводе-подтверждении
+        brief_force - кортеж имен переменных, значение которых необходимо
+                 отобразите в выводе-подтверждении, даже если параметр является
+                 несовместимым
+        expert_label - текст вместо стандартной надписи для отображения
+                 расширенных параметров
+        next_label - название кнопки для запуска действия или перехода к
+                 следующей группе
+        custom_buttons - список кортежей (название кнопки, текст на кнопке,
+                                          значение (метод), элемент ("button")
         """
         default_next_label = _("Next")
         default_expert_label = _("Click for advanced settings")
@@ -1359,12 +1397,14 @@ class DataVars(SimpleDataVars):
              'hide':hide,
              'image':image,
              'brief':brief,
+             'custom_buttons':custom_buttons,
              'brief_force':brief_force,
              'expert_label':expert_label or default_expert_label,
              'next_label':next_label or default_next_label})
         lastIndex = len(self.groups)-1
         for var in chain(normal,expert):
             self.mapVarGroup[var] = lastIndex
+        return self
 
     def getGroups(self):
         """
@@ -1455,6 +1495,8 @@ class DataVars(SimpleDataVars):
         """
         Check variables in group or all in info
         """
+        if not self.groups:
+            return []
         errors = []
         if self.groups:
             keys = self.mapVarGroup.keys()
@@ -1504,6 +1546,36 @@ class DataVars(SimpleDataVars):
                             self.Check(var,self.Get(var))
                     else:
                         val = self.fixWsdlTableValue(varObj,val)
+                        if varObj.type in ("choice","choice-list"):
+                            choicedata = self.ChoiceAndComments(var)
+                            if any(choicedata):
+                                if choicedata[1] is None:
+                                    choicedata = [choicedata[0],
+                                                  choicedata[0]]
+                                choicedata = zip(*choicedata)
+                                if varObj.type == "choice":
+                                    vals = [val]
+                                else:
+                                    vals = val
+                                res = []
+                                for val in vals:
+                                    if not val in map(lambda x:x[0],choicedata):
+                                        result = filter(lambda x:(
+                                                    _toUNICODE(val).lower() in _toUNICODE(x[0]).lower() or \
+                                                    _toUNICODE(val).lower() in _toUNICODE(x[1]).lower()),
+                                                 choicedata)
+                                        if len(result) > 1:
+                                            raise VariableError(_("Ambiguous choice:%s")%
+                                                ",".join(map(
+                                                lambda x:'"%s"(%s)'%(x[1],x[0]) if x[0] != x[1] else x[0],
+                                                             result)))
+                                        elif result:
+                                            val = result[0][0]
+                                    res.append(val)
+                                if varObj.type == "choice":
+                                    val = res[0]
+                                else:
+                                    val = res
                         self.Set(var,val)
                         # raise error for atempt set uncompatible variable
                         if uncomperr:
