@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2008-2012 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import platform
 import re
 from calculate.lib.datavars import Variable,VariableError,ReadonlyVariable
 from calculate.lib.utils.files import readFile,process, listDirectory,isMount
-from calculate.lib.utils.device import getUdevDeviceInfo
+from calculate.lib.utils.device import getUdevDeviceInfo,loadEfiVars
 from calculate.lib.utils.common import getKernelUid
 
 class VariableOsArchMachine(ReadonlyVariable):
@@ -133,3 +133,24 @@ class VariableOsScratch(ReadonlyVariable):
             return "on" if isMount('/mnt/scratch/workspace') else "off"
         else:
             return "on" if isMount('/mnt/scratch') else "off"
+
+class VariableOsUefiSet(ReadonlyVariable):
+    """
+    Current loaded system
+    """
+    def get(self):
+        if not path.exists('/sys/firmware/efi'):
+            loadEfiVars()
+        if path.exists('/sys/firmware/efi'):
+            return "on"
+        else:
+            return "off"
+
+class VariableClUsedAction(ReadonlyVariable):
+    """
+    Переменная содержит все найденные при обработке шаблонов
+    событийные переменные (ac_)
+    """
+    type = "table"
+    def get(self):
+        return []
